@@ -579,15 +579,24 @@ extern int dwarf_bitoffset (Dwarf_Die *die);
 /* Return array order attribute of DIE.  */
 extern int dwarf_arrayorder (Dwarf_Die *die);
 
-/* Return source language attribute of DIE.  */
-extern int dwarf_srclang (Dwarf_Die *die);
+/* Return DW_LANG source language of CU DIE.
+   Returns the DW_LANG constant on success, -1 otherwise.  */
+extern int dwarf_srclang (Dwarf_Die *cudie);
+
+/* Provides the DW_LNAME source language and version of the given CU
+   DIE.  LVERSION may be NULL. Returns zero on success.  */
+extern int dwarf_language (Dwarf_Die *cudie,
+			   Dwarf_Word *lname,
+			   Dwarf_Word *lversion) __nonnull_attribute__ (2);
 
 
 /* Get abbreviation at given offset for given DIE.  */
 extern Dwarf_Abbrev *dwarf_getabbrev (Dwarf_Die *die, Dwarf_Off offset,
 				      size_t *lengthp);
 
-/* Get abbreviation at given offset in .debug_abbrev section.  */
+/* Get abbreviation at given offset in .debug_abbrev section.  On
+   success return zero and fills in ABBREVP.  When there is no (more)
+   abbrev at offset returns one.  On error returns a negative value.  */
 extern int dwarf_offabbrev (Dwarf *dbg, Dwarf_Off offset, size_t *lengthp,
 			    Dwarf_Abbrev *abbrevp)
      __nonnull_attribute__ (4);
@@ -701,6 +710,15 @@ extern int dwarf_linediscriminator (Dwarf_Line *line, unsigned int *discp)
 extern const char *dwarf_linesrc (Dwarf_Line *line,
 				  Dwarf_Word *mtime, Dwarf_Word *length);
 
+/* Return the caller of this line if inlined.  If not inlined,
+   return NULL.  */
+extern Dwarf_Line *dwarf_linecontext (Dwarf_Lines *lines, Dwarf_Line *line);
+
+/* Return the function name in this line record. If this line is
+   inlined, this is the name of the function that was inlined. If this line
+   is not inlined, return NULL.  */
+extern const char *dwarf_linefunctionname (Dwarf *dbg, Dwarf_Line *line);
+
 /* Return file information.  The returned string is NULL when
    an error occurred, or the file path.  The file path is either absolute
    or relative to the compilation directory.  See dwarf_decl_file.  */
@@ -818,13 +836,20 @@ extern int dwarf_getlocation_attr (Dwarf_Attribute *attr,
    For DW_TAG_array_type it can apply much more complex rules.  */
 extern int dwarf_aggregate_size (Dwarf_Die *die, Dwarf_Word *size);
 
-/* Given a language code, as returned by dwarf_srclan, get the default
-   lower bound for a subrange type without a lower bound attribute.
-   Returns zero on success or -1 on failure when the given language
-   wasn't recognized.  */
+/* Given a DW_LANG language code, as returned by dwarf_srclang, get
+   the default lower bound for a subrange type without a lower bound
+   attribute.  Returns zero on success or -1 on failure when the given
+   language wasn't recognized.  */
 extern int dwarf_default_lower_bound (int lang, Dwarf_Sword *result)
   __nonnull_attribute__ (2);
 
+/* Given a DW_LNAME language code, as returned by dwarf_language, get
+   the default lower bound for a subrange type without a lower bound
+   attribute.  Returns zero on success or -1 on failure when the given
+   language wasn't recognized.  */
+extern int dwarf_language_lower_bound (Dwarf_Word lname, Dwarf_Sword *result)
+  __nonnull_attribute__ (2);
+
 /* Return scope DIEs containing PC address.
    Sets *SCOPES to a malloc'd array of Dwarf_Die structures,
    and returns the number of elements in the array.
@@ -1072,6 +1097,33 @@ extern int dwarf_frame_register (Dwarf_Frame *frame, int regno,
   __nonnull_attribute__ (3, 4, 5);
 
 
+/* Return offset and/or size of CU's contribution to SECTION in a
+   DWARF package file.
+
+   If CU is not from a DWARF package file, the file does not have
+   SECTION, or CU does not contribute to SECTION, then *OFFSETP and
+   *SIZEP are set to 0 (this is not an error and the function will
+   return 0 in that case).
+
+   SECTION is a DW_SECT section identifier.  Note that the original
+   GNU DWARF package file extension for DWARF 4 used slightly
+   different section identifiers.  This function uses the standardized
+   section identifiers and maps the GNU DWARF 4 identifiers to their
+   standard DWARF 5 analogues: DW_SECT_LOCLISTS (5) refers to
+   .debug_locs.dwo for DWARF 4.  DW_SECT_MACRO (7) refers to
+   .debug_macinfo.dwo for DWARF 4 or .debug_macro.dwo for the GNU
+   .debug_macro extension for DWARF 4 (section identifier 8 is
+   DW_SECT_RNGLISTS in DWARF 5, NOT DW_SECT_MACRO like in the GNU
+   extension.)  .debug_types.dwo does not have a DWARF 5 equivalent,
+   so this function accepts the original DW_SECT_TYPES (2).
+
+   Returns 0 for success or -1 for errors reading the DWARF package
+   file data or if an unknown SECTION constant is given.  OFFSETP and
+   SIZEP may be NULL.  */
+extern int dwarf_cu_dwp_section_info (Dwarf_CU *cu, unsigned int section,
+				      Dwarf_Off *offsetp, Dwarf_Off *sizep);
+
+
 /* Return error code of last failing function call.  This value is kept
    separately for each thread.  */
 extern int dwarf_errno (void);
