@@ -75,6 +75,7 @@ ebl_core_note_type_name (Ebl *ebl, uint32_t type, char *buf, size_t len)
 	    KNOWNSTYPE (PPC_VMX);
 	    KNOWNSTYPE (PPC_SPE);
 	    KNOWNSTYPE (PPC_VSX);
+	    KNOWNSTYPE (PPC_TM_SPR);
 	    KNOWNSTYPE (386_TLS);
 	    KNOWNSTYPE (386_IOPERM);
 	    KNOWNSTYPE (X86_XSTATE);
@@ -91,12 +92,18 @@ ebl_core_note_type_name (Ebl *ebl, uint32_t type, char *buf, size_t len)
 	    KNOWNSTYPE (ARM_HW_BREAK);
 	    KNOWNSTYPE (ARM_HW_WATCH);
 	    KNOWNSTYPE (ARM_SYSTEM_CALL);
+	    KNOWNSTYPE (ARM_TAGGED_ADDR_CTRL);
+	    KNOWNSTYPE (ARM_SVE);
+	    KNOWNSTYPE (ARM_SSVE);
+	    KNOWNSTYPE (ARM_ZA);
+	    KNOWNSTYPE (ARM_PAC_ENABLED_KEYS);
+	    KNOWNSTYPE (ARM_PAC_MASK);
 	    KNOWNSTYPE (SIGINFO);
 	    KNOWNSTYPE (FILE);
 #undef KNOWNSTYPE
 
 	  default:
-	    snprintf (buf, len, "%s: %" PRIu32, gettext ("<unknown>"), type);
+	    snprintf (buf, len, "%s: %" PRIu32, _("<unknown>"), type);
 
 	    res = buf;
 	  }
