@@ -19,7 +19,7 @@
 
 testfiles testfile36 testfile36.debug
 
-testrun_compare ./dwflmodtest -e testfile36 <<\EOF
+testrun_compare ${abs_builddir}/dwflmodtest -e testfile36 <<\EOF
 module:                                00000000..00002308 testfile36 (null)
 module:                                00000000 DWARF 0 (no error)
 module:                                00000000..00002308 testfile36 testfile36.debug
